/*
 * wmdimp
 * Copyright (c) 2003 Daniele Tamino <aslo@wmdimp.cjb.net>.
 * This software covered by the GPL.  See COPYING file for details.
 *
 * player_mad.c
 *
 * mp3 file player using libmad and output.c to play sound (header file).
 *
 */
#ifndef mad_playerH
#define mad_playerH

#include "player.h"
#include "thrcomm.h"
#include <mad.h>

enum msg_tx_value {
    MSG_NONE,
    MSG_TERM, /* terminate decoder thread */
    MSG_STOP, /* stop playing and wait for next song (used?) */
    MSG_PAUSE,
    MSG_RESUME, /* resume if paused, else nothing */
    MSG_SONGCHANGE, /* input file has changed, do what's to do */
    MSG_SEEK /* look at dec_seek to get new position */
};

enum msg_rx_value {
   ACK_NONE,
   ACK_PAUSE, /* in response to MSG_PAUSE */
   ACK_ERROR, /* an error occurred (used handling SEEK,SONGCHANGE) */
   ACK_DONE /* decoder handled the request */   
};

typedef struct player_mad
{
    player pl;

    /* song info  */
    song_pos pos, len; /* position and length */
    long bitrate; /* bitrate (in bits per second) */
    pthread_mutex_t pos_mtx; /* locks position */
    /* no need to lock len and bitrate since these are modified only by
       scan_thread: just pthread_join it */

    player_cb_func callback;
    void* cb_param;

    /* input file data */
    char* filename; /* input file name (needed for scan_thread) */
    char* title; /* title (get from ID3 or something) */
    pthread_mutex_t title_mtx;
    int infd; /* input file descriptor  */
    time_t mtime; /* last modif. time when opened */
    long flen;
    int eof;
    int mmap; /* == 1 if we used mmap */
#ifdef HAVE_MMAP
    void* buffer; /* mmap'ed file address */
#endif
    pthread_mutex_t infile_mtx;

    pthread_t scan_thr;
    volatile int scan_thr_running; /* ==1 while scan_thr is running */
    volatile int abort_scan; /* signals scan_thr to abort */
    pthread_t dec_thr;
#if 1
    thrchannel dec_thr_chn; /* channel to communicate with dec_thr */
#else
    volatile int dec_thr_running; /* ==1 while dec_thr is running */
    /* Simple communication scheme between main and decoder thread:
         Main locks dec_msg_mtx, puts a value in dec_msg_tx, then signals
       dec_msg_cond and (if needed) waits on dec_msg_cond for decoder to put
       its acknoledge in dec_msg_rx, in the end it releases dec_msg_mtx.
         Decoder, when it sees the msg, it does all its work locking
       dec_msg_mtx, then when it's done it sets its acknoledge in dec_msg_rx,
       sets dec_msg_tx=MSG_NONE, signals dec_cond and releases dec_msg_mtx.
       Seems to work :-D (hopefully) !
       ********************
       Well, it doesn't :(
       Trying to use semaphores, that seems to be The Right Choice.
     */
    /* message from main to decoder thread */
    volatile enum msg_tx_value dec_msg_tx;
    volatile song_pos dec_seek; /* where to seek when MSG_SEEK is set */
    /* message/ack from decoder to main thread */
    volatile enum msg_rx_value dec_msg_rx;
    sem_t dec_msg_rx_sem;
#endif
    pthread_mutex_t my_mtx; /* mutex on changing myself */
} player_mad;

extern player_factory player_mad_factory;

#endif
