#ifndef stringH
#define stringH

/* simple string framework */
typedef struct string_part {
    char *s;
    int len;
    struct string_part* next;
} string_part;

typedef struct string {
    string_part* head;
    string_part* tail;
} string;

static inline string* string_init(string* str)
{
    assert(str);
    str->head = str->tail = NULL;
    return str;
}

static inline void string_finish(string* str)
{
    string_part *p = str->head;
    while(p) {
        string_part* t = p->next;
        if(p->s) free(p->s);
        free(p);
        p = t;
    }
#ifndef NDEBUG
    str->head = str->tail = NULL;
#endif
}

static inline string* string_new()
{
    return string_init(malloc(sizeof(string)));
}

static inline void string_free(string* str)
{
    string_finish(str);
    free(str);
}

static inline string* string_nappend(string* str, const char* s, int len)
{
    string_part *n = malloc(sizeof(string_part));
    assert(n);
    n->s = malloc((len + 1) * sizeof(char));
    n->len = len;
    assert(n->s);
    memcpy(n->s, s, len + 1);   
    n->next = NULL;
    if(str->tail) {
        str->tail->next = n;
        str->tail = n;
    } else
        str->tail = str->head = n;
    return str;
}

static inline string* string_append(string* str, const char* s)
{
    return string_nappend(str, s, strlen(s));
}

static inline char* string_getbytes(string* str)
{
    int totlen = 0;
    string_part* p;
    char *s, *t;
    for(p = str->head; p; p = p->next)
        totlen += p->len;
    s = malloc((totlen + 1) * sizeof(char));
    assert(s);
    t = s;
    for(p = str->head; p; p = p->next) {
        memcpy(t, p->s, p->len);
        t += p->len;
    }
    *t = '\0';
    return s;
}


#endif
