/*
 * wmdimp
 * Copyright (c) 2003 Daniele Tamino <aslo@wmdimp.cjb.net>
 * This software covered by the GPL.  See COPYING file for details.
 *
 * testout.c
 *
 * Program to test the output module
 *
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <unistd.h>
#include <getopt.h>

#include "parsecfg.h"
#include "output.h"

static char* cfg_name = "testoutrc";

output_fmt fmt, out_fmt;

enum { BUFFER_SIZE=10000 };

volatile sig_atomic_t do_pause = 0; /* pause is already defined :(((  */
volatile sig_atomic_t term = 0;

void signal_term(int sig) {
    (void) sig;
    term = 1;
}

void signal_pause(int sig) {
    do_pause = !do_pause;
    signal(sig, signal_pause);
}

int main(int argc, char** argv)
{
    FILE* in = stdin;
    opt_hash ht;
    char* endp;
    void* buffer;
    int bps; /* bytes per sample */
    int n, ret, c;

    /* fill in with default values */
    fmt.rate = 44100;
    fmt.chan = 2;
    fmt.bits = 16;

    while((c = getopt(argc, argv, "f:r:c:b:")) >= 0) {
    switch(c) {
    case 'f': /* load another config file */
        cfg_name = optarg;
        break;
    case 'r': /* set input data rate */
        n = strtol(optarg, &endp, 0);
        if(endp == optarg || *endp != '\0') {
        fprintf(stderr, "setting rate: %s is not an integer!\n",
            optarg);
        } else {
        if(n <= 0)
            fprintf(stderr, "setting rate: %d is not > 0!\n", n);
        else
            fmt.rate = n;
        }
        break;
    case 'c': /* set input data channels */
        n = strtol(optarg, &endp, 0);
        if(endp == optarg || *endp != '\0') {
        fprintf(stderr, "setting channels: %s is not an integer!\n",
            optarg);
        } else {
        if(n <= 0)
            fprintf(stderr, "setting channels: %d is not > 0!\n", n);
        else
            fmt.chan = n;
        }
        break;
    case 'b': /* set input data bits per channel */
        n = strtol(optarg, &endp, 0);
        if(endp == optarg || *endp != '\0') {
        fprintf(stderr, "setting rate: %s is not an integer!\n",
            optarg);
        } else
        if(n <= 0 || n > 32)
            fprintf(stderr, "setting bits: %d is not > 0 and <= 32\n", n);
        else
            fmt.bits = n;
        break;
    }
    }

    bps = ((fmt.bits + 7) >> 3) * fmt.chan;
    buffer = malloc(bps * BUFFER_SIZE);
    if(!buffer) {
    fprintf(stderr, "out of memory (%d bytes)!\n", bps * BUFFER_SIZE);
    return -1;
    }
    printf("loading options from '%s'... ", cfg_name); fflush(stdout);
    opt_hash_init(&ht);
    ret = load_config_file(cfg_name, output_specs.spec, &ht,
               CFL_SEARCHWD/*|CFL_IGNUNKNOWN*/);
    if(ret != CFGERR_NONE)
    fprintf(stderr, "\nError loading '%s': %s\n",
        cfg_name, get_cfg_err(ret));
    else
    printf("done!\n");

    /* now do the test */
    ret = output_init(&ht);
    opt_hash_free(&ht); /* we don't need them anymore */
    if(!ret) {
    fprintf(stderr, "cannot init output module\n");
    return 1;
    }

    printf("Reading input at %ld Hz, %d bits, %d channels.\n",
       fmt.rate, fmt.bits, fmt.chan);
    ret = output_open(&fmt, &out_fmt);
    if(!ret) {
    fprintf(stderr, "cannot open output device\n");
    return 2;
    }

    printf("Opened output device at %ld Hz, %d bits, %d channels.\n",
       out_fmt.rate, out_fmt.bits, out_fmt.chan);

    /* set some signals to (somehow) control playback */
    signal(SIGINT, signal_term);
    signal(SIGTERM, signal_term);
    signal(SIGUSR1, signal_pause);

    while(!term && !feof(in) && !ferror(in)) {
    if(do_pause) {
        printf("Pause\n");
        output_pause();
        while(do_pause && !term)
        sleep(1);
        printf("Resume\n");
    } else if(!term) {
        n = fread(buffer, bps, BUFFER_SIZE, in);
        output_write(buffer, n);
    }
    }
    printf("Stopping audio... "); fflush(stdout);
    output_stop(1, 0);
    printf("Closing audio device... "); fflush(stdout);
    output_close();
    printf("bye!\n");
    return 0;
}
