/*
 * wmdimp
 * Copyright (c) 2003 Daniele Tamino <aslo@wmdimp.cjb.net>
 *
 * Based on wmmp3: Copyright (c)1999 Patrick Crosby <xb@dotfiles.com>.
 *
 * This software covered by the GPL.  See COPYING file for details.
 *
 * utils.h
 *
 * various utility functions used by many functions
 *
 */
#ifndef utilsH
#define utilsH

#include <ctype.h>
#include <assert.h>
#include "parsecfg.h"

/* get file extension */
char * getext(const char* fname);
/* get filename (with ext) from complete path */
char * getname(const char* fname);

static inline char* skipspaces(char* s)
{
    while(isspace(*s)) s++;
    return s;
}

/* simple string reader with ' and " style quoting and \ escaping,
 * term is a list of terminators, such as punctation charters, etc.
 * term may be NULL, so only spaces terminates the string.
 * term should not contain quoting (',") or escaping (\) characters.
 */
char* read_string(char* src, char** end, char* term);

/* strip trailing spaces from s */
void stripspace(char *s);


/* utils for string lists */
typedef struct str_list {
    char* str;
    struct str_list* next;
} str_list;

/* free all the content of l and l itself */
void free_str_list(str_list* l);

/* load a string list from a configuration option tree */
void load_str_list(opt_tree_node* array, str_list** lst);

/* macros for pthread condition's */
typedef void (*cleanup_f)(void*);

#define WAIT_COND(cond_name) \
  {\
    pthread_cleanup_push((cleanup_f)pthread_mutex_unlock, \
                         (void*)&cond_name##_mtx); \
    pthread_cond_wait(&cond_name##_cond, &cond_name##_mtx); \
    pthread_cleanup_pop(1); \
  }

#define WAIT_COND_UNTIL(cond, cond_name) \
  {\
    pthread_cleanup_push((cleanup_f)pthread_mutex_unlock, \
                         (void*)&cond_name##_mtx); \
    while(!(cond)) { \
      pthread_cond_wait(&cond_name##_cond, &cond_name##_mtx); \
    } \
    pthread_cleanup_pop(1); \
  }

#endif
