/*
 * wmdimp
 * Copyright (c) 2003 Daniele Tamino <aslo@wmdimp.cjb.net>
 *
 * This software covered by the GPL.  See COPYING file for details.
 *
 * vect.h
 *
 * dynamically growing vector (all inline functions, no .c file)
 *
 */
#ifndef vectH
#define vectH

#include <stdlib.h>

enum { vect_min = 8, vect_grow = 2 };

struct vect {
    int alloc;   /* allocated storage */
    int size;    /* num of elements */
    int elem_sz; /* element size */
    void* v;     /* the vector */
};

#ifdef NO_INLINE_VECT

int vect_init_sz(struct vect* v, int elem_sz, int n);
int vect_init(struct vect* v, int elem_sz);
int vect_incr(struct vect* v, int inc);
void* vect_get_last(struct vect* v);

#else

static inline int vect_init_sz(struct vect* v, int elem_sz, int n)
{
    v->v = malloc(n * elem_sz);
    if(!v->v) return 0;
    v->alloc = n;
    v->elem_sz = elem_sz;
    v->size = 0;
    return 1;
}

static inline int vect_init(struct vect* v, int elem_sz) {
    return vect_init_sz(v, elem_sz, vect_min);
}

static inline void* vect_get_last(struct vect* v)
{
    return (char*) v->v + (v->size - 1) * v->elem_sz;
}

/* fast macro version of vect_get_last */
#define _vect_get_last(vect, type) ( ((type*)(vect)->v) + (vect)->size - 1 )

static inline int vect_incr(struct vect* v, int inc)
{
    v->size += inc;
    if(v->size >= v->alloc) {
    void* np;
        do {
            v->alloc *= vect_grow;
        } while(v->size >= v->alloc);
        np = realloc(v->v, v->alloc * v->elem_sz);
        if(!np) return 0;
    v->v = np;
    }
    return 1;
}

#endif /* NO_INLINE_VECT */

#endif
